/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package compodslimport.importWizards;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.ListIterator;
import java.util.Map;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.uml2.uml.Model;

import compodslimport.Activator;
import compodslimport.CompoDSLParser;

public class ImportWizard extends Wizard implements IImportWizard
{

	ImportWizardPage mainPage;

	public ImportWizard()
	{
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	public boolean performFinish()
	{
		final IPath containerPath = mainPage.getContainerFullPath();
		final IFile file = mainPage.createNewFile();

		if (file == null)
		{
			return false;
		}
		IRunnableWithProgress op = new IRunnableWithProgress()
		{
			public void run(IProgressMonitor monitor)
					throws InvocationTargetException
			{
				try
				{
					doFinish(containerPath, file, monitor);
				}
				catch (CoreException e)
				{
					throw new InvocationTargetException(e);
				}
				finally
				{
					monitor.done();
				}
			}
		};
		try
		{
			getContainer().run(true, false, op);
		}
		catch (InterruptedException e)
		{
			return false;
		}
		catch (InvocationTargetException e)
		{
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error",realException.getMessage());
			return false;
		}

		return true;
	}

	protected void doFinish(IPath containerPath, IFile file,IProgressMonitor monitor) throws CoreException
	{
		// TODO Auto-generated method stub
		String modelfilename = "model.uml"; // Kompatibilitt mit papyrus
		monitor.beginTask("Creating Model", 2);
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		IResource res = root.findMember(containerPath);

		if (!res.exists() || !(res instanceof IContainer))
		{
			String mesg = "Container \"" + containerPath.toString()
					+ "\" does not exist.";
			IStatus status = new Status(IStatus.ERROR, Activator.getId(),
					IStatus.OK, mesg, null);
			throw new CoreException(status);
		}
		IContainer container = (IContainer) res;
		IResource found = container.findMember(modelfilename);

		ResourceSet resourceSet = new ResourceSetImpl();
		org.eclipse.uml2.uml.Package pkg = null;
		CompoDSLParser parser = new CompoDSLParser(resourceSet);
		IFile modelfile = null;
		String modelpath = null;
		URI modelfileURI = null;
		if (found != null)
		{
			modelfile = (IFile) found;
			modelpath = modelfile.getFullPath().toString();
			modelfileURI = URI.createPlatformResourceURI(modelpath, true);
			Resource modelresource = resourceSet.getResource(modelfileURI, true);
			EList<EObject> content = modelresource.getContents();
			ListIterator<EObject> it = content.listIterator();
			EObject e = it.next();
			Model umodel = (Model) e;
			pkg = parser.setModel(umodel);
		}
		else
		{
			pkg = parser.createNewModelAndPackage();
			modelfile = container.getFile(new Path(modelfilename));
		}

		Boolean result = parser.parse(pkg, file);
		if (!result)
		{
			String mesg = "Error: Can't Parse " + file.getName();
			IStatus status = new Status(IStatus.ERROR, Activator.getId(),
					IStatus.OK, mesg, null);
			throw new CoreException(status);
		}

		if (modelfileURI == null)
		{
			modelpath = modelfile.getFullPath().toString();
			modelfileURI = URI.createPlatformResourceURI(modelpath, true);
		}
		Resource emfResource = resourceSet.createResource(modelfileURI);
		Model model = parser.getModel();
		if (model != null)
		{
			emfResource.getContents().add(model);
			// Save the contents of the resource to the file
			// system.
			//
			Map<?, ?> options = new HashMap<String, String>();
			try
			{
				emfResource.save(options);
			}
			catch (IOException ioe)
			{
				IStatus status = new Status(IStatus.ERROR, Activator.getId(),
						IStatus.OK, "An error occured during saving resource",
						ioe);
				throw new CoreException(status);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 * org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection)
	{
		setWindowTitle("CompoDSL Import Wizard"); // NON-NLS-1
		setNeedsProgressMonitor(true);
		mainPage = new ImportWizardPage("Import CompoDSL", selection); // NON-NLS-1
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages()
	{
		super.addPages();
		addPage(mainPage);
	}

}
